<?php
/**
 * Open Url Test Class
 *
 * PHP version 5
 *
 * Copyright (C) Leipzig University Library 2017.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category Finc
 * @package  Tests
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development:testing:unit_tests Wiki
 */
namespace FincTest\View\Helper\Root;

use finc\View\Helper\Root\OpenUrl;

/**
 * Open Url Test Class
 *
 * @category Finc
 * @package  Tests
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development:testing:unit_tests Wiki
 */
class OpenUrlTest extends \VuFindTest\View\Helper\Root\OpenUrlTest
{
    const ISSN      = 456;
    const OPEN_URL  = "open=url";
    const ZDB_ID    = 123;

    public function testAddCustomParamsNoParams()
    {
        $instance = $this->getInstance();
        $nonConfigParams = "key1=value1&foo=bar&key2=value2";
        $this->assertEquals("{$nonConfigParams}", $instance->addCustomParams('Ezb', $nonConfigParams));
        $this->assertEquals("{$nonConfigParams}", $instance->addCustomParams('Redi', $nonConfigParams));
    }

    public function testAddCustomStaticParamsForEzb()
    {
        $config = $this->getBaseConfigObject();
        $config->General->active_resolvers = 'Ezb, Redi';
        $config->Ezb->custom_params = ['foo:bar'];
        $instance = $this->getInstance($config);
        $nonConfigParams = "key1=value1&key2=value2";
        $this->assertEquals("{$nonConfigParams}&foo=bar", $instance->addCustomParams('Ezb', $nonConfigParams));
        $this->assertEquals("{$nonConfigParams}", $instance->addCustomParams('Redi', $nonConfigParams));
    }

    public function testDONTOverrideExistingParamsForEzb()
    {
        $config = $this->getBaseConfigObject();
        $config->General->active_resolvers = 'Ezb, Redi';
        $config->Ezb->custom_params = ['foo:bar'];
        $instance = $this->getInstance($config);
        $nonConfigParams = "key1=value1&foo=bar&key2=value2";
        $this->assertEquals("{$nonConfigParams}", $instance->addCustomParams('Ezb', $nonConfigParams));
        $this->assertEquals("{$nonConfigParams}", $instance->addCustomParams('Redi', $nonConfigParams));
    }

    public function testAddCustomStaticParamsForRedi()
    {
        $config = $this->getBaseConfigObject();
        $config->General->active_resolvers = 'Ezb, Redi';
        $config->Redi->custom_params = ['foo:bar'];
        $instance = $this->getInstance($config);
        $nonConfigParams = "key1=value1&key2=value2";
        $this->assertEquals("{$nonConfigParams}", $instance->addCustomParams('Ezb', $nonConfigParams));
        $this->assertEquals("{$nonConfigParams}&foo=bar", $instance->addCustomParams('Redi', $nonConfigParams));
    }

    public function testDONTOverrideExistingParamsForRedi()
    {
        $config = $this->getBaseConfigObject();
        $config->General->active_resolvers = 'Ezb, Redi';
        $config->Redi->custom_params = ['foo:bar'];
        $instance = $this->getInstance($config);
        $nonConfigParams = "key1=value1&foo=bar&key2=value2";
        $this->assertEquals("{$nonConfigParams}", $instance->addCustomParams('Ezb', $nonConfigParams));
        $this->assertEquals("{$nonConfigParams}", $instance->addCustomParams('Redi', $nonConfigParams));
    }

    public function testAddCustomDynamicParamsForEzb()
    {
        $config = $this->getBaseConfigObject();
        $config->General->active_resolvers = 'Ezb, Redi';
        $config->Ezb->custom_params = ['zdbid:getZdbId'];
        $instance = $this->getInstance($config);
        $nonConfigParams = "key1=value1&foo=bar&key2=value2";

        $this->assertEquals(
            "{$nonConfigParams}&zdbid=" . self::ZDB_ID,
            $instance->addCustomParams('Ezb', $nonConfigParams)
        );
        $this->assertEquals("{$nonConfigParams}", $instance->addCustomParams('Redi', $nonConfigParams));
    }

    public function testAddCustomDynamicParamsForRedi()
    {
        $config = $this->getBaseConfigObject();
        $config->General->active_resolvers = 'Ezb, Redi';
        $config->Redi->custom_params = ['issn:getCleanISSN'];
        $instance = $this->getInstance($config);
        $nonConfigParams = "key1=value1&foo=bar&key2=value2";

        $this->assertEquals(
            "{$nonConfigParams}&issn=" . self::ISSN,
            $instance->addCustomParams('Redi', $nonConfigParams)
        );
        $this->assertEquals("{$nonConfigParams}", $instance->addCustomParams('Ezb', $nonConfigParams));
    }

    public function testValidateParamsEzbFalse()
    {
        $this->rulesConfig['Ezb'] = [0 => ['include' => ["methods" => "getSourceID"]]];
        $instance = $this->getInstance();

        $this->assertFalse($instance->isActive());
    }

    public function testValidateParamsEzbTrue()
    {
        $this->rulesConfig['Ezb'] = [0 => ['include' => ["methods" => "getSourceID"]]];
        $instanceWithIssn = $this->getInstance(false, true, false);
        $this->assertTrue($instanceWithIssn->isActive());

        $instanceWithZdbId = $this->getInstance(false, false, true);
        $this->assertTrue($instanceWithZdbId->isActive());

        $instanceWithIssnAndZdbId = $this->getInstance(false, true, true);
        $this->assertTrue($instanceWithIssnAndZdbId->isActive());
    }

    public function testIsActiveIsIdempotent()
    {
        $this->rulesConfig['Ezb'] = [0 => ['include' => ["methods" => "getSourceID"]]];
        $instance = $this->getInstance(false, true, false);

        $instance->isActive();
        $activeResolversFirst = $instance->getActiveResolvers();
        $resolverCountFirst = $instance->getRecordResolvers();
        $areaFirst = $instance->getArea();

        $instance->isActive();
        $activeResolversSecond = $instance->getActiveResolvers();
        $resolverCountSecond = $instance->getRecordResolvers();
        $areaSecond = $instance->getArea();

        $this->assertEquals(
            $activeResolversFirst,
            $activeResolversSecond,
            sprintf(
                "Active resolvers must not change on recall, first %s then %s.",
                implode(", ", $activeResolversFirst),
                implode(", ", $activeResolversSecond)
            )
        );

        $this->assertEquals(
            $resolverCountFirst,
            $resolverCountSecond,
            sprintf(
                "Resolvers fetched by current rules must not change on recall, first %s then %s.",
                implode(", ", $resolverCountFirst),
                implode(", ", $resolverCountSecond)
            )
        );

        $this->assertEquals(
            $areaFirst,
            $areaSecond,
            sprintf(
                "Context for same record must not change on recall, first %s then %s.",
                $areaFirst,
                $areaSecond
            )
        );
    }

    /*
        Helper Functions
    */

    /**
     * @param bool $config
     * @param bool $recordHasIssn
     * @param bool $recordHasZdbId
     * @return \finc\View\Helper\Root\OpenUrl
     */
    protected function getInstance($config = false, $recordHasIssn = false, $recordHasZdbId = false)
    {
        if (empty($config)) {
            $config = $this->getBaseConfigObject();
        }
        $driver = $this->getBasicFincMockDriver('finc\RecordDriver\SolrDefault', $recordHasIssn, $recordHasZdbId);
        $context = new \VuFind\View\Helper\Root\Context();
        $openUrlClazz = new OpenUrl($context, $this->rulesConfig, $config);
        $openUrlClazz->__invoke($driver, "results");

        return $openUrlClazz;
    }

    /**
     * Get mock driver that returns an openURL.
     *
     * @param string $class Class to mock
     *
     * @param bool $hasIssn
     * @param bool $hasZdbId
     * @return \PHPUnit_Framework_MockObject_MockObject
     */
    protected function getBasicFincMockDriver(
        $class = 'finc\RecordDriver\SolrDefault',
        $hasIssn = false,
        $hasZdbId = false
    ) {
        $driver = $this->getMockBuilder($class)
            ->disableOriginalConstructor()->getMock();

        $driver->method('tryMethod')
            ->will($this->returnCallback(array($this, 'getCallback')));

        $driver->method('getOpenUrl')
                ->willReturn(self::OPEN_URL);

        if ($hasIssn) {
            $driver->method('getCleanISSN')
                ->willReturn(self::ISSN);
        }

        if ($hasZdbId) {
            $driver->method('getZdbId')
                ->willReturn(self::ZDB_ID);
        }

        return $driver;
    }

    public function getCallback($foo)
    {
        if ($foo == "getZdbId") {
            return self::ZDB_ID;
        }

        if ($foo == "getCleanISSN") {
            return self::ISSN;
        }

        return null;
    }

    protected function getBaseConfigObject()
    {
        $openUrlConfig = [
            'OpenURL' => [
                'url' => "http://services.d-nb.de/fize-service/gvr/full.xml",
                'rfr_id' => "www.ub.uni-leipzig.de",
                'resolver' => "ezb",
                'window_settings' => "toolbar=no",
                'show_in_results' => false,
                'show_in_record' => false,
                'show_in_holdings' => true,
                'embed' => true,
                'replace_other_urls' => true
            ],
        ];

        $config = [
            'General' => [
                'active_resolvers' => 'Ezb, Redi'
            ],
            'Redi' => [
                'url' => 'http://www.redi-bw.de/links/whz',
                'resolver' => 'redi',
            ],
            'Ezb' => [
                'url' => 'http://services.dnb.de/fize-service/gvr/full.xml',
                'resolver' => 'ezb',
            ],
            'openUrlConfig' => $openUrlConfig
        ];

        return $this->arrayToObject($config);
    }

    public function arrayToObject($array)
    {
        $obj = new \stdClass();
        foreach ($array as $k => $v) {
            if (strlen($k)) {
                if (is_array($v)) {
                    $obj->{$k} = $this->arrayToObject($v); //RECURSION
                } else {
                    $obj->{$k} = $v;
                }
            }
        }
        return $obj;
    }
}
